#include "header.h"
#include <csignal>
#include <ctime>
#include <limits>
#include <string>
#ifdef AUTOTEST
#include <random>
std::string generateIdea(const std::vector<std::vector<std::string>>& templ) {
    static std::random_device rd;
    static std::mt19937 gen(rd());
    std::string idea;
    for (char a=0;a<templ.size();a++) {
        std::uniform_int_distribution<> dist(0, templ[a].size() - 1);
        if(a) idea += " ";
        idea += templ[a][dist(gen)];
    }
    return idea;
}
#else
#include <iostream>
#endif
int main(int argc, char* argv[]) {
  std::fstream board(BOARDFILE);
  sem_t *sem = sem_open(SEM_NAME, 0);
  if (sem == SEM_FAILED) {
    perror("Не вдалося відкрити семафор! Чи науковий кервіник онлайн?");
    return 1;
  }

  int fd = open(FIFO_NAME, O_WRONLY);
  if (fd == -1) {
    perror("Не вдалося відкрити іменовану трубк-... Але вдалося відкрити семафор? Дивно. Більше так не робіть.");
    sem_close(sem);
    return 1;
  }
#ifdef AUTOTEST
  const std::vector<std::vector<std::string>> ideatemplate = {
      {"негайно", "терміново", "безкоштовно", "невідкладно", "добровільно", "примусово", "заповзято", "агресивно", "непоштиво", "канонічно"},
      {"створити", "написати", "розробити", "придумати", "уявити", "зобразити", "намалювати", "спроектувати", "зруйнувати", "спалити"},
      {"штучний", "м'який", "чорний", "літаючий", "мокрий", "газований", "цифровий", "свіжий", "прозорий", "новий", "потужний"},
      {"проект", "сервер", "міксер", "органайзер", "степлер", "дірокол", "скутер", "унітаз", "акваріум", "хобот", "ВНС"},
      {"під водою", "в космосі", "в Житомирі", "у волинських болотах", "на Говерлі", "в Кривому Розі", "в Чернівцях", "в пустелі", "на хмарі", "в стратосфері"},
      {"для студентів", "для операторів", "для астрологів", "для шахтарів", "для сантехніків", "для ховрахів", "для в'єтнамських поросят", "для роботів - гітаристів", "для піонерів", "для мотоциклістів", "задля благу"},
      {"денної форми навчання", "машинного доїння", "глибокого буріння", "гарячого тестування", "підводного плавання", "хімічного синтезу", "неперервної виплавки", "швидкісної їзди", "приємного гойдання", "нічних польотів"}
  };
#endif
  auto startTime = std::time(nullptr);
  std::vector<std::string> generatedIdeas;
  
#ifdef AUTOTEST
  while (std::difftime(std::time(nullptr), startTime) < BRAINSTORM_TIME) {
      std::string newIdea = generateIdea(ideatemplate);
      generatedIdeas.push_back(newIdea);
      sleep(1); //to be removed
 }
#else
  std::string newIdea;
  pid_t pid=fork();
  if(pid)
    {
      sleep(BRAINSTORM_TIME);
      kill(pid, SIGTERM);
      std::cout<<"\rЧас сплив! Натисніть Enter, щоб продовжити...";
      std::cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
    }
    else{
  while (std::difftime(std::time(nullptr), startTime) < BRAINSTORM_TIME) {
    std::cout << "Ідея: ";
    std::getline(std::cin, newIdea);
    sem_wait(sem);
    board.seekp(0, std::ios::end);
    board << newIdea << std::endl;
    sem_post(sem);
  }
    }
  std::cout<<"Vote for the best idea!" <<std::endl;
  std::string line;
  short pos=1;
  char ideacount=0;
  board.seekg(0, std::ios::beg);
  while (std::getline(board, line)) {
    std::cout << pos<< ". " << line << std::endl;
    pos++;
    ideacount++;
  }
  short myvotes=MAX_VOTES;
  std::cout<<"You might vote for one idea multiple times"<<std::endl;
  while(myvotes) {
    do{
    std::cout<<"You got "<<myvotes<<" votes left! Vote for the best idea!"<<std::endl;
    std::cin>>newIdea;
    for (char b=0;b<newIdea.size();b++)if(newIdea[b]<48||newIdea[b]>57)std::cout<<"Not a number!"<<std::endl;continue;
    if(stoi(newIdea)>ideacount){std::cout<<"No such idea!"<<std::endl;continue;}
    }while(0);
    myvotes--;
    sem_wait(sem);
    write(fd, newIdea.c_str(), newIdea.size());
    sem_post(sem);
  }
  board.close();
#endif
  close(fd);
  return 0;
}
